﻿#-------------------------------------------------------------------------#
# Script_Name : Test-SmtpConnection
# Description : Emulate telnet SMTP test connection
# Date : July 2020
# Written by : Griffon 
#               
# WebSite :http://www.c-nergy.be - http://www.c-nergy.be/blog
# Version : 1.0
#
# Notes: Fork from initial script of Glen Scales
#
# Disclaimer : Script provided AS IS. Use it at your own risk....
# You can use this script and distribute it as long as credits are kept 
# in place and unchanged 
#-------------------------------------------------------------------------#


<#
 .Synopsis
  Emulate Telnet connection to Exchange Server using Powershell. 

 .Description
  Mimic telnet SMTP Commands (HELO/MAIL FROM/RCPT TO/DATA..) and provide response returned by the mail server
  using exclusively PowerShell technology. Alternative solution if Telnet Client not installed or not available

 .Link
  http://c-nergy.be/blog/?p=15365
  https://www.c-nergy.be/products.html

 .Parameter RemoteHost
  Server or IP address of the Exchange/SMTP Server.

 .Parameter Port
  Port to connect To.  Default Port is 25.

 .Parameter From
  Email address of the Sender.

 .Parameter To
  Email address of the Recipient. 

 .Parameter Subject
  Default Subject set to Test Message + Time.  Can provide your own Subject for the message.

 .Parameter HEL
  Default is set to HELO but if preferred EHLO value can be passed.

 .Example
  Minium Parameters to initiate smtp connection. 
  Test-SmtpConnection -RemoteHost 10.10.10.10  -From demo@test.com -To Target@test.com

 .Example
    Set a custom Subject Message.
    Test-SmtpConnection -RemoteHost 10.10.10.10  -From demo@test.com -To Target@test.com -Subject "This is a test Message"

 .Example
    Use EHLO instead of HELO.
    Test-SmtpConnection -RemoteHost 10.10.10.10  -From demo@test.com -To Target@test.com -Subject "This is a test Message" -HEL EHLO
#>






function Test-SmtpConnection {
param(
    [Parameter(Mandatory=$true)] [String] $remotehost,
    [Parameter(Mandatory=$true)] [String] $From,
    [Parameter(Mandatory=$true)] [String] $To,
    $port="25",
    [String] $Subject="Test Message $(get-date)",
    [String] $HEL="HELO"
    )

# -Default BODY MESSAGE USED WHEN MESSAGE IS SEND                         
$testMsg=@"
DATA
Subject: $Subject

This is a test Message

.

"@

function readResponse {

while($stream.DataAvailable)  
   {  
      $read = $stream.Read($buffer, 0, 1024)    
      write-host -n -foregroundcolor cyan ($encoding.GetString($buffer, 0, $read))  
      ""
   } 
}

#------------------------------------#
# CODE TO PERFORM SMTP CONNECTION    #
#------------------------------------#

$socket = new-object System.Net.Sockets.TcpClient($remoteHost, $port) 

if($socket -eq $null) { return; } 

$stream = $socket.GetStream() 
$writer = new-object System.IO.StreamWriter($stream) 
$buffer = new-object System.Byte[] 1024 
$encoding = new-object System.Text.AsciiEncoding 
readResponse($stream)

#-------------------------------------------------------#
# SMTP COMMAND HELO OR EHLO - DEFAULT IS HELO           #
#-------------------------------------------------------#


write-host -foregroundcolor yellow $HEL
""
$writer.WriteLine($HEL) 
$writer.Flush()
start-sleep -m 500 
readResponse($stream)


#---------------------------------------------------#
# SMTP COMMAND Mail FROM                            #
#---------------------------------------------------#


$command = "MAIL FROM: $from"  
write-host -foregroundcolor yellow "MAIL FROM: $from"  
$writer.WriteLine($command) 
$writer.Flush()
start-sleep -m 500 
readResponse($stream)


#---------------------------------------------------#
# SMTP COMMAND RCPT TO                              #
#---------------------------------------------------#

$command = "RCPT TO: $To"
write-host -foregroundcolor yellow $command
$writer.WriteLine($command) 
$writer.Flush()
start-sleep -m 500 
readResponse($stream)

#---------------------------------------------------#
# SMTP COMMAND DATA                                 #
#---------------------------------------------------#
write-host -foregroundcolor yellow $testMsg
$writer.WriteLine($testMsg) 
$writer.Flush()
start-sleep -m 500 
readResponse($stream)

#---------------------------------------------------#
# CLOSING SMTP CONNECTION                           #
#---------------------------------------------------#

$command = "QUIT" 
write-host -foregroundcolor yellow $command
""
$writer.WriteLine($command) 
$writer.Flush()
start-sleep -m 500 
readResponse($stream)
## Close the streams 
$writer.Close() 
$stream.Close() 

}



Export-ModuleMember -Function Test-SmtpConnection


